C
C    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
C    -------------------------------------------------------------------

C    This file is part of Wavlib.

C    Wavlib is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.

C    Wavlib is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


C     Portable function to determine file size. It's a bit stupid,
C     but the FORTRAN standard does not define any way
C     to determine file sizes.
C     If the file does not exist, return -1, if it's too long
C     return -2.

C     U is a free file unit.

      INTEGER*8 FUNCTION FSIZE(U,FNAME)
        IMPLICIT NONE
        CHARACTER*(*) FNAME
        INTEGER MAXWAV
        INTEGER U, IOS
        COMMON /ERR/ IOS
        BYTE T,P
        INTEGER*8 START,ENDING, I

        IOS = 0
        OPEN(U, FILE=FNAME, STATUS='OLD', ACCESS='DIRECT',
     +  FORM='UNFORMATTED', RECL=1, IOSTAT=IOS, ERR=100)

C       Binary search for file end.
C       If we read fine, we're below file end,
C       if we get an error, we're above file end.

        START = 0
C       The + 1 might be a bit cryptic. It's there because
C       we return START, and START does never reach ENDING.
C       We assume we're on a 2's complement machine, so everything's
C       fine.
        ENDING = MAXWAV()

  10    IF (ENDING - START .LE. 1) GOTO 30
        I = (START + ENDING)/2
        READ(U,REC=I,IOSTAT=IOS) T
        IF (IOS .NE. 0) GOTO 20
C       We read without error, so FSIZE .GE. I
        START = I
        GOTO 10

C       Error encounted, assume it's end of file (unfortunately, 
C       lots of IOS codes mean EOF)
  20    CONTINUE
C       IOSTAT is 3, so end of file encountered, and FSIZE .LT. I
        ENDING = I
        GOTO 10

C       Found the position of the last byte of the file
  30    FSIZE = START

C       Now, this method can only find out if the file
C       is less than or equal to MAXWAV() - 1.
C       If we get that value, see if we can read more of the file.

        IF (FSIZE .GE. MAXWAV() - 1) THEN
          READ(U,REC=MAXWAV(),IOSTAT=IOS) T
          IF (IOS .NE. 0) THEN
                FSIZE = MAXWAV() - 1
          ELSE
                CLOSE(U)
C               Work with blocksize 2:
                OPEN(U, FILE=FNAME, STATUS='OLD', ACCESS='DIRECT',
     +          FORM='UNFORMATTED', RECL=2)
                
                I = MAXWAV()/2
                IF(I*2 .EQ. MAXWAV()) THEN
C                   Mawvav() is even, read two blocks
                    READ(U,REC=I,IOSTAT=IOS) T,P
                ELSE
C                   Mawvav() is odd, read one byte after I:
                    READ(U,REC=I+1,IOSTAT=IOS) T
                ENDIF 
C               If we read with error, the file is exactly
C               MAXWAV() bytes long, otherwise it's longer,
C               so return an error:
                IF (IOS .EQ. 0) THEN
                        FSIZE = MAXWAV()
                ELSE
                        FSIZE = -2
                ENDIF
          ENDIF
        ENDIF

        CLOSE(U)
        RETURN
 100    FSIZE = -1
        RETURN
      END


