//
//    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
//    -------------------------------------------------------------------

//    This file is part of Wavlib.

//    Wavlib is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    Wavlib is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License
//    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


// There is no standard way of shrinking file lengths in Fortran 77.
// Here is provided some code to do that in C on different systems,
// POSIX and Win32. Add code to your own system (and add a new
// #ifdef block) if your system is not POSIX or Win32 compliant.

// a few flags to enable support for large files:

#define _FILE_OFFSET_BITS 	64
#define _LARGEFILE64_SOURCE 	1
#define _LARGE_FILES

#include <stdio.h>

#include <sys/types.h>

#ifdef gnu
#define posix
#endif

#ifdef posix
	#include <unistd.h>
#elif defined(win)
	#include <fcntl.h>
	#include <sys/stat.h>
	#include <io.h>
#else
	#error "Specify system (eg. make SYSTEM=posix or make SYSTEM=win)"
#endif

#ifndef FINT
	#error "Specify what integer (long long int or int) Fortran gives. (make FINT=blabla)."
#endif

// set file lenght of file by name *name to size *SIZE.
// *name is a FORTRAN string of length len. (len passed by FORTRAN)
//
int setsiz_(char *name, FINT *size, int len) {
	int i, lastpos, val;

#ifdef posix
	off_t s;
	s = (off_t) *size;
#elif defined(win)
	int fh;
#endif

//	Find out the actual length of the name (FORTRAN pads remaining
//	string with ' '):
	for (i=len-1;i>0 && name[i] == ' ';i--);
	lastpos = i+1;
	// null terminate file name
	name[lastpos] = '\0';

#ifdef posix
	val = truncate(name, s);
#elif defined(win)

if( (fh = _open( name, _O_RDWR | _S_IREAD | _S_IWRITE ))  != -1 )
   {
      val = _chsize( fh, (long int) *size );
      _close( fh );
   } else {
 	val = -1;
   }

#else
//	Here should perhaps be implemented a function
//	to read entire file up to *size into another, empty file
//	and switch names on the two files.
//	For now, we just issue an error.
	printf("Wavlib: Error: No system specified at compiletime.");
	val = -1;
#endif
	// un-nullterminate the string again:
	name[lastpos] = ' ';
return val;
}

