C
C    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
C    -------------------------------------------------------------------

C    This file is part of Wavlib.

C    Wavlib is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.

C    Wavlib is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


C     Close a WAV file opened by OPENW.
C     This routine flushes all buffers
C     and updates metadata to the file,
C     does the final shrinking of too long files
C     and finally closes the file.
C     DO NOT FORGET TO CALL THIS ROUTINE AFTER USING THE
C     LIBRARY TO WRITE WAV FILES OR THEY WILL BE CORRUPT!
      SUBROUTINE CLOSEW(U)
        IMPLICIT NONE
        INTEGER U, I
        INCLUDE "../../common/wav.f"
        CHARACTER FNAME*(NAMLEN), FNAME2*(NAMLEN+10)
        INTEGER SETSIZ
        INTEGER LOGLEN

        IF (DESCR(DESLEN,U) .LT. 0) THEN
C       File not opened!
        WRITE(*,*) 'Wavlib: Warning: trying to close unopened file: ',U
        RETURN
        ENDIF

C       Check if the length of subchunks and file
C       matches what's written in header data, fix if not:

        CALL UPDLEN(U)

C       Dispose all buffers beloning to the file unit
        DO 10 I = 1, NBUFS
                IF (RECORD(1,I) .EQ. U) THEN
                        CALL DEPORT(I)
                ENDIF
  10    CONTINUE

C       The Fortran 77 standard does not define what happens
C       when we do not write a complete record at the end of a file.
C       If the compiler writes additional data to the
C       open file on write, we need to cut the
C       rest of the file off. This is handled with a C function:

        IF (FILEWR(U)) THEN
         INQUIRE(UNIT=U,NAME=FNAME)
         INQUIRE(UNIT=U,NAME=FNAME2)

         IF (FNAME .NE. FNAME2) THEN
           WRITE(*,*) 'Wavlib: Warning: Filename ', FNAME,
     +     'is too long. It will not be cut, and may be corrupted.'
            FILEWR(U) = .FALSE.
         ENDIF
        ENDIF

C       We call some I/O routines from C, so it's desireable
C       to have all data on disk before we truncate (in SETSIZ)

        CALL FLUSH(U)
C       This call to FLUSH is likely unnessecary, as CLOSE
C       probably does flush stuff immediately. If the
C       compiler does not recognice FLUSH(), try removing the call.

        CLOSE(U)

        IF (FILEWR(U)) THEN
            IF(SETSIZ(FNAME,DESCR(DESLEN,U)) .NE. 0) THEN
              WRITE(*,*) 'Wavlib: Warning: Could not cut file ',
     +        FNAME(1:LOGLEN(FNAME)), ', it may be corrupted.'
            ENDIF
          FILEWR(U) = .FALSE.
        ENDIF

C       Clear the descriptor table entry to avoid confusion
C       if accidently trying to read a closed WAV file.

        DO 20 I = 1, NDESCR
                DESCR(I,U) = -1
  20    CONTINUE

        RETURN
      END


