C
C    Copyright (C) 2009 Anders Aspegren Søndergaard <andersas@gmail.com>
C    -------------------------------------------------------------------

C    This file is part of Wavlib.

C    Wavlib is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.

C    Wavlib is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with Wavlib. If not, see <http://www.gnu.org/licenses/>.


C     Subroutine to shorten an open wave file.
C     It takes a file unit U and a number SAM,
C     being the number of samples to keep.
C     If the file has fewer than SAM samples (per channel)
C     it the routine does nothing.
C     Returns the same as OPENW, or -2 on bad input and -3 on
C     too long a filename, and -4 if the file has
C     extra subchunks after the data subchunk, which could not
C     be moved.

      INTEGER FUNCTION SHORTN(U,SAM)
        INTEGER U, SAM
        INTEGER I, POS, MAXREC
        CHARACTER FNAME*500, FNAME1*400
        INCLUDE "../../common/wav.f"
        LOGICAL MOVDAT

C       Check if the file is open
        IF (DESCR(DESLEN,U) .LE. 0) THEN
          WRITE(*,*) 'Wavlib: Warning: trying to shorten unopened file.'
          SHORTN = -2
          RETURN
        ENDIF

C       In case we get insane values:
        IF (SAM .LT. 0) THEN
                SHORTN = -2
                RETURN
        ENDIF

        IF (SAM .GE. DESCR(DESSAM,U)) THEN
C               No need to cut anything away, return success.
                SHORTN = 0
                RETURN
        ENDIF

C       Get file names, and check if they're too big for us:
        INQUIRE(UNIT=U,NAME=FNAME)
        INQUIRE(UNIT=U,NAME=FNAME1)
        IF (FNAME .NE. FNAME1) THEN
                SHORTN = -3
                RETURN
        ENDIF
        
C       The operation will fail miserably if the data
C       subchunk is not the last subchunk in the file.
C       (since we're cutting away the rest of the file)
C       If it isn't, rearrange the file:
        IF (.NOT. DTLAST(U)) THEN
               IF(.NOT. MOVDAT(U)) THEN
                        SHORTN = -4
                        RETURN
               ENDIF
        ENDIF

C       Calculate the position in file to cut away from
        POS = DESCR(DESDAT,U) + 8
     +      + SAM*DESCR(DESBPS,U)/8*DESCR(DESCNS,U)

C       Update DESLEN, DESDLN and DESSAM descriptors
        DESCR(DESSAM,U) = SAM
        DESCR(DESDLN,U) = SAM * DESCR(DESCNS,U)*DESCR(DESBPS,U)/8
        DESCR(DESLEN,U) = DESCR(DESDAT,U) + DESCR(DESDLN,U) + 8
C       Mark the file length and data subchunk length as
C       being changed.
        CHGWLN(U) = .TRUE.
        CHGSAM(U) = .TRUE.

C       Note that there's no reason to fear dirty buffers
C       being written past the end of new file length
C       (COMMIT sees to that), but we might still
C       have buffers in memory that contain old values.
C       They won't be read either (GETSTR sees to that)
C       unless something is written past the old values.
C       Any buffers left containing data beyond the new end of file
C       should either be reinitialized or in this case simply unloaded.
C       Record MAXREC or greater should be deported:

        MAXREC = (DESCR(DESLEN,U) - 1) / BUFLEN + 1 

        DO 10 I = 1, NBUFS
          IF (RECORD(1,I) .EQ. U .AND. RECORD(2,I) .GE. MAXREC) THEN 
                CALL DEPORT(I)
          ENDIF
  10    CONTINUE

C       Return 0 on success:
        SHORTN = 0

        RETURN
      END
